;--------------------------------------------------------------------------
; Testing program for DUART 1.0
;
; The DUART card is the device developped by the IDE64 Project team.
; For more information: http://ide64.come.to
;
; 5.12.1999 (c) Josef Soucek
; 19.3.2000     Tomas Pribyl
;
;--------------------------------------------------------------------------

; This program uses DUART XR68C681 in LOCAL LOOPBACK MODE.
; Transmit and receive baudrate is set to 115200.
; Random byte from BASIC ROM is sended to channel A, received back and 
; compared. If received byte is different, border color is changed. 


duart    = $de00    ;COM1,2 BASE ADDRESS
;duart   = $de10    ;COM3,4 BASE ADDRESS


                    .include "kernal.asm"

;--------------------------------------------------------------------------
;DUART XR68C681 registers

MR1A     = duart    ;Mode register, channel A(1)
MR2A     = duart    ;Mode register, channel A(2)
SRA      = duart+1  ;Status register, channel A
CSRA     = duart+1  ;Clock select register, channel A
MISR     = duart+2  ;Masked interrupt status register
CRA      = duart+2  ;Command register, channel A
RHRA     = duart+3  ;Rx holding register, channel A
THRA     = duart+3  ;Tx holding register, channel A
IPCR     = duart+4  ;Input port change register
ACR      = duart+4  ;Auxiliary control register
ISR      = duart+5  ;Interrupt status register
IMR      = duart+5  ;Interrupt mask register
CTU      = duart+6  ;Counter/timer Upper byte register
CTL      = duart+7  ;Counter/timer Lower byte register


MR1B     = duart+8  ;Mode register, channel B(1)
MR2B     = duart+8  ;Mode register, channel B(2)
SRB      = duart+9  ;Status register, channel B
CSRB     = duart+9  ;Clock select register, channel B
CRB      = duart+10 ;Command rgister, channel B
RHRB     = duart+11 ;Rx holding register, channel B
THRB     = duart+11 ;Tx holding register, channel B
IVR      = duart+12 ;Interrupt vector register
IP       = duart+13 ;Input port
OPCR     = duart+13 ;Output port configuration register
SCC      = duart+14 ;Start counter/timer command
SOPBC    = duart+14 ;Set output port bit command
STC      = duart+15 ;Stop counter/timer command
COPBC    = duart+15 ;Clear output port bits 1 command


;--------------------------------------------------------------------------

               *= $1000

               jsr set_indicator   ; set LED ON
               jsr set_tr_rate     ; set Receive/Transmit rate
               jsr set_mode        ; set MODE

               lda #%00000101      ;enable Tx,Rx
               sta CRA
               ;
         
MainLoop       jsr TestStatus      ;Test status & PrintERROR
             ;  ldy $a2             ;read time
             ;  lda $a000,y         ;read random data from BASIC ROM
               
               inc OutData
             ;  sta OutData
               lda OutData
               jsr SendByte
               jsr ReceiveByte
               sta InData
               sta $0400
               
               lda InData
               cmp OutData         ;DATA OK
               beq MainLoop
               
               lda OutData
               jsr PrintHex
               lda #$20
               jsr ChrOut
               lda InData
               jsr PrintHex
               lda #$0d
               jsr ChrOut
               
               inc $d020           ;data ERROR
Wait4Key       jsr $ffe4
               beq Wait4key
               jmp MainLoop







;-----------------------------------------------------------------------------
; routine TEST STATUS and IDENTIFY ERROR
;         
;-----------------------------------------------------------------------------
TestStatus     lda SRA
               and #%11110000     ;test ERROR
               bne ErrorDetect
               clc
               rts
               
ErrorDetect    lda #%10000000
               bit SRA
               bne RecBrkErr
               
               lda #%01000000
               bit SRA
               bne FramingErr
               
               lda #%00100000
               bit SRA
               bne ParityErr
               
               lda #<OverErrTxt
               ldy #>OverErrTxt
PrintStrErr    jsr $ab1e
               lda #%01000000
               sta CRA            ;reset ERROR indicator
               sec
               rts
               
OverErrTxt     .Text "Overrun ERROR",$0d,0

RecBrkErr      lda #<RecBrkTxt
               ldy #>RecBrkTxt
               jmp PrintStrErr
RecBrkTxt      .Text "Received BREAK",$0d,0               
               
               
FramingErr     lda #<FramingTxt
               ldy #>FramingTxt
               jmp PrintStrErr
FramingTxt     .Text "Framing ERROR",$0d,0               
               
ParityErr      lda #<ParityTxt
               ldy #>ParityTxt
               jmp PrintStrErr
ParityTxt      .Text "Parity ERROR",$0d,0                                            
               



;-----------------------------------------------------------------------               
; routine RECEIVE BYTE (non IRQ transfer)
;               
;-----------------------------------------------------------------------         
ReceiveByte    lda SRA
               and #%00000001  ;RXRDY
               beq ReceiveByte

               lda RHRA
               rts


;-----------------------------------------------------------------------
; routine SEND BYTE (non IRQ transfer)
;
;-----------------------------------------------------------------------
SendByte       pha
SendBusy       lda SRA
               and #%00000100
               beq SendBusy
               pla
               sta THRA
               rts


;-----------------------------------------------------------------------
; Set mode registers
;
set_mode = *
               lda #%00010000   ;Reset MR pointer
               sta CRA
               ;
               lda #%00010011   ;8bit no parity
               sta MR1A
               lda #%10000111   ;1 stop bit, Local LOOPBACK mode
               sta MR2A
         
               lda #%00100000   ;Reset Receiver
               sta CRA
               lda #%00110000   ;Reset Transmitter
               sta CRA
               rts


;-----------------------------------------------------------------------
; Set input/output transfer rate to 115200baud
;
set_tr_rate = *
;              lda #%10110000  ;transmit X=0
               lda #%10100000  ;tr X=1
               sta CRA
;              lda #%10010000  ;receiver X=0
               lda #%10000000  ;rx X=1
               sta CRA
               ;
               lda #%10000000  ;Bit R.Set #2
               sta ACR
               ;
               lda #%10001000  ;2400/115.2
               sta CSRA        ;l=tx h=r
               rts



;-----------------------------------------------------------------------
; Set indicator channel A
; - LED ON
;
set_indicator = *
               lda #%01010000
               sta OPCR
               rts
         


OutData       .byte 0
InData        .byte 0

              .include "printnum.asm"
